package explorviz.hpc_monitoring.record;

import java.io.Serializable;
import java.util.Arrays;
import explorviz.hpc_monitoring.record.events.AbstractOperationEvent;

public final class Trace implements Serializable, IRecord {
    private static final long                 serialVersionUID = 8589405631073291022L;

    private final TraceMetadata               traceMetadata;
    private final AbstractOperationEvent[]    traceEvents;
    private final RuntimeStatisticInformation runtimeInformation;

    public Trace(final TraceMetadata traceMetadata,
            final AbstractOperationEvent[] traceEvents) {
        this.traceMetadata = traceMetadata;
        this.traceEvents = traceEvents;
        runtimeInformation = new RuntimeStatisticInformation(1); // TODO
    }

    public TraceMetadata getTraceMetadata() {
        return traceMetadata;
    }

    public AbstractOperationEvent[] getTraceEvents() {
        return traceEvents;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder(64);
        sb.append(super.toString());
        sb.append("\n\tTrace: ");
        sb.append(traceMetadata);
        for (final AbstractOperationEvent traceEvent : traceEvents) {
            sb.append("\n\t");
            sb.append(traceEvent.getClass().getSimpleName());
            sb.append(": ");
            sb.append(traceEvent);
        }
        sb.append("\n\t");
        sb.append(runtimeInformation.getClass().getSimpleName());
        sb.append(": ");
        sb.append(runtimeInformation);
        sb.append('\n');
        return sb.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = (prime * result)
                + ((traceMetadata == null) ? 0 : traceMetadata.hashCode()); // NOCS
        // (?:)
        result = (prime * result) + Arrays.hashCode(traceEvents);
        return result;
    }

    @Override
    public boolean equals(final Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (this.getClass() != obj.getClass()) {
            return false;
        }
        final Trace other = (Trace) obj;
        if (traceMetadata == null) {
            if (other.traceMetadata != null) {
                return false;
            }
        }
        else if (!traceMetadata.equals(other.traceMetadata)) {
            return false;
        }
        if (!Arrays.equals(traceEvents, other.traceEvents)) {
            return false;
        }
        return true;
    }

    public RuntimeStatisticInformation getRuntime() {
        return runtimeInformation;
    }
}
